<?php
/**
 * A general file class
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2012 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
class BaseFile extends BaseFrameworkSystem {
	/**
	 * The current file we are working in
	 */
	private $fileName = '';

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct ($className) {
		// Call parent constructor
		parent::__construct($className);
	}

	/**
	 * Destructor for cleaning purposes, etc
	 *
	 * @return	void
	 */
	public final function __destruct() {
		// Try to close a file
		$this->closeFile();

		// Call the parent destructor
		parent::__destruct();
	}

	/**
	 * Setter for file name
	 *
	 * @param	$fileName	The new file name
	 * @return	void
	 */
	protected final function setFileName ($fileName) {
		$fileName = (string) $fileName;
		$this->fileName = $fileName;
	}

	/**
	 * Getter for file name
	 *
	 * @return	$fileName	The current file name
	 */
	public final function getFileName () {
		return $this->fileName;
	}

	/**
	 * Initializes this file class
	 *
	 * @param	$fileName	Name of this abstract file
	 * @return	void
	 */
	protected function initFile ($fileName) {
		// Get a file i/o pointer instance
		$pointerInstance = ObjectFactory::createObjectByConfiguredName('file_raw_input_output_class', array($fileName));

		// ... and set it here
		$this->setPointerInstance($pointerInstance);
	}

	/**
	 * Close a file source and set it's instance to null and the file name
	 * to empty
	 *
	 * @return	void
	 * @todo	~10% done?
	 */
	public function closeFile () {
		$this->partialStub('Unfinished method.');

		// Remove file name
		$this->setFileName('');
	}

	/**
	 * Determines seek position
	 *
	 * @return	$seekPosition	Current seek position
	 */
	public final function determineSeekPosition () {
		return $this->getPointerInstance()->determineSeekPosition();
	}

	/**
	 * Seek to given offset (default) or other possibilities as fseek() gives.
	 *
	 * @param	$offset		Offset to seek to (or used as "base" for other seeks)
	 * @param	$whence		Added to offset (default: only use offset to seek to)
	 * @return	$status		Status of file seek: 0 = success, -1 = failed
	 */
	public function seek ($offset, $whence = SEEK_SET) {
		return $this->getPointerInstance()->seek($offset, $whence);
	}

	/**
	 * Size of this file
	 *
	 * @return	$size	Size (in bytes) of file
	 * @todo	Handle seekStatus
	 */
	public function size () {
		return $this->getPointerInstance()->size();
	}

	/**
	 * Read data a file pointer
	 *
	 * @return	mixed	The result of fread()
	 * @throws	NullPointerException	If the file pointer instance
	 *									is not set by setPointer()
	 * @throws	InvalidResourceException	If there is being set
	 */
	public function readFromFile () {
		return $this->getPointerInstance()->readFromFile();
	}

	/**
	 * Reads given amount of bytes from file.
	 *
	 * @param	$bytes	Amount of bytes to read
	 * @return	$data	Data read from file
	 */
	public function read ($bytes) {
		return $this->getPointerInstance()->read($bytes);
	}

	/**
	 * Write data to a file pointer
	 *
	 * @param	$dataStream		The data stream we shall write to the file
	 * @return	mixed			Number of writes bytes or FALSE on error
	 * @throws	NullPointerException	If the file pointer instance
	 *									is not set by setPointer()
	 * @throws	InvalidResourceException	If there is being set
	 *											an invalid file resource
	 */
	public function writeToFile ($dataStream) {
		return $this->getPointerInstance()->writeToFile($dataStream);
	}

	/**
	 * Rewinds to the beginning of the file
	 *
	 * @return	$status 	Status of this operation
	 */
	public function rewind () {
		return $this->getPointerInstance()->rewind();
	}
}

// [EOF]
?>
